--[[---------------------------------------------------------------------------------
 ADDON DECLARATION && SAVED VARIABLES && MENU INITIALIZATION
------------------------------------------------------------------------------------]]
SacredBuff = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")
local L = AceLibrary("AceLocale-2.2"):new("SacredBuff")
local BS = AceLibrary("Babble-Spell-2.2")
local G = AceLibrary("Gratuity-2.0")
local D = AceLibrary("Dewdrop-2.0")

local majV = "2"
local minV = "$Rev: 15475 $"
SacredBuff.version = majV.."."..string.sub(minV, -7, -7).."."..string.sub(minV, -6, -2)

-- BINDINGS
-----------------------------------------------------------
SACREDBUFF_BINDING_HEADER = L["SacredBuff"]
BINDING_NAME_MOUNT = L["Mount"]

-- SAVED VARIABLES
-----------------------------------------------------------
SacredBuff:RegisterDB("SBDB", "SBPCDB")

SacredBuff:RegisterDefaults("profile", {
	movable = true,
	tooltip = 2,
	toggle = true
})

SacredBuff:RegisterDefaults("char", {
	groupMode = true,
	candleCount = 0,
	tablet = {},
	spells = {
		[ BS["Inner Fire"] ] = false,
		[ BS["Resurrection"] ] = false, 
		
		[ BS["Divine Spirit"] ] = false,
		[ BS["Power Word: Fortitude"] ] = false,
		[ BS["Shadow Protection"] ] = false,
		
		[ BS["Prayer of Spirit"] ] = false,
		[ BS["Prayer of Fortitude"] ] = false,
		[ BS["Prayer of Shadow Protection"] ] = false
	}
})

-- MENU INITIALIZATION
-----------------------------------------------------------
SacredBuff:RegisterChatCommand({"/sacredbuff", "/sb"}, {
	type = "group",
	args = {
		[ L["Lock"] ] = {
			name = L["Lock"], type = "toggle",
			desc = L["(Un)Lock the SacredBuff frame."],
			map = { [false] = "|cffff0000"..L["On"].."|r", [true] = "|cff00ff00"..L["Off"].."|r" },
			get = function()
				return SacredBuff.db.profile.movable
			end,
			set = function()
				SacredBuff.db.profile.movable = not SacredBuff.db.profile.movable
				if ( SacredBuff.db.profile.movable ) then
					SacredBuff.frame.title:Show()
					SacredBuff.frame:SetBackdropColor(0.00, 0.00, 0.00, 0.60)
				else
					SacredBuff.frame.title:Hide()
					SacredBuff.frame:SetBackdropColor(0.00, 0.00, 0.00, 0.00)
				end
			end
		},
		[ L["Tooltip"] ] = {
			name = L["Tooltip Options"], type = "group",
			desc = L["Select what information the tooltip provides."],
			args = {
				[ L["None"] ] = {
					name = L["No Tooltip"], type = "execute",
					desc = L["No tooltip will be displayed over the spell icons."],
					func = function()
						if ( SacredBuff.db.profile.tooltip ~= 0 ) then
							SacredBuff.db.profile.tooltip = 0
							SacredBuff:Print(L["Tooltips turned off."])
						end
					end
				},
				[ MANA ] = {
					name = L["Mana Only"], type = "execute",
					desc = L["Only the mana cost of the spell will be displayed in its tooltip."],
					func = function()
						if ( SacredBuff.db.profile.tooltip ~= 1 ) then
							SacredBuff.db.profile.tooltip = 1
							SacredBuff:Print(L["Tooltips will now display the spell's mana cost."])
						end
					end
				},
				[ L["Full"] ] = {
					name = L["Full Tooltip"], type = "execute",
					desc = L["The spell's full tooltip will be displayed on mouseover."],
					func = function()
						if ( SacredBuff.db.profile.tooltip ~= 2 ) then
							SacredBuff.db.profile.tooltip = 2
							SacredBuff:Print(L["Full spell tooltips will now be displayed."])
						end
					end
				}
			},
			disabled = true
		}
	}
})


--[[---------------------------------------------------------------------------------
 LOCAL FUNCTIONS
------------------------------------------------------------------------------------]]
local function findSpell(spell)
	local i = 1
	while true do
		local sName = GetSpellName(i, BOOKTYPE_SPELL)
		if ( not sName ) then
			return false
		end
		if ( sName == spell ) then
			return true
		end
		i = i + 1
	end
	return false
end

local function talentfinder(x, y, s)
	local tName, _, _, _, cRank = GetTalentInfo(x, y)
	assert(tName == s, string.format(L["Talent_Error"], x, y, s))
	if (( cRank ) and ( cRank > 0 )) then
		SacredBuff.db.char.talent = s
		return true
	end
	if (( SacredBuff.db.char.talent ) and ( SacredBuff.db.char.talent == s )) then
		SacredBuff.db.char.talent = nil
		SacredBuff.frame.talent:Hide()
	end
	return false
end

local function finditem(itemId, literal, count, list)
	if ( literal ) then
		itemId = "item:"..itemId..":"
	end
	local c = 0
	local tbl
	if ( list ) then
		tbl = {}
	end
	for bag = 0, NUM_BAG_FRAMES do
		for slot = 1, GetContainerNumSlots(bag) do
			local itemLink = GetContainerItemLink(bag, slot)
			if ( itemLink ) then
				if ( string.find(GetContainerItemLink(bag, slot), itemId) ) then
					if ( list ) then
						table.insert(tbl, itemId)
					elseif ( count ) then
						local _, c2 = GetContainerItemInfo(bag, slot)
						c = c + c2
					else
						return bag, slot
					end
				end
			end
		end
	end
	if ( list ) then
		return tbl
	elseif ( count ) then
		return c
	end
end

local function click(spell, onSelf)
	if ( onSelf == "LeftButton" ) then
		onSelf = nil
	end
	CastSpellByName(BS[spell], onSelf)
end

local function ismounted()
	G:Erase()
	local c = 0
	while ( GetPlayerBuff(c, "HELPFUL|HARMFUL|PASSIVE") ~= -1 ) do
		local i = GetPlayerBuff(c, "HELPFUL|HARMFUL|PASSIVE")
		G:SetPlayerBuff(i)
		if ( G:Find(L["Increases speed by"]) ) then
			return i
		end
		c = c + 1
	end
end

local function deleteTable(tbl, ret)
	for k, _ in pairs(tbl) do
		if ( type(tbl[k]) == "table" ) then
			deleteTable(tbl[k])
		else
			tbl[k] = nil
		end		
	end
	if ( ret ) then
		return tbl
	end
	tbl = nil
end


--[[---------------------------------------------------------------------------------
 ADDON INITIALIZATION && ACTIVATION
------------------------------------------------------------------------------------]]
function SacredBuff:OnInitialize()
	self.inCombat = false
	
	if (( self.version ~= self.db.char.version ) or ( not self.db.char.version )) then
		local race, talent
		if ( self.db.char[RACE] ) then
			race = self.db.char[RACE]
		end
		if ( self.db.char.talent ) then
			talent = self.db.char.talent
		end
		
		self:ResetDB("char")
		
		if ( race ) then
			self.db.char[RACE] = race
		end
		if ( talent ) then
			self.db.char.talent = talent
		end
		
		self.db.char.version = self.version
	end
	
	self:CreateMainFrame()
	
--	Figure out racial spell, add it to table here so that we
--	don't waste memory on spells the player will never learn.
	if ( not self.db.char[RACE] ) then
		local r = UnitRace("player")
		if ( UnitFactionGroup("player") == "Alliance" ) then
			if (( r == "Dwarf" ) and ( findSpell(BS["Fear Ward"]) )) then
				self.db.char[RACE] = BS["Fear Ward"]
			elseif (( r == "NightElf" ) and ( findSpell(BS["Elune's Grace"]) )) then
				self.db.char[RACE] = BS["Elune's Grace"]
			elseif ( findSpell(BS["Feedback"]) ) then
				self.db.char[RACE] = BS["Feedback"]
			end
		else
			if (( r == "Scourge" ) and ( findSpell(BS["Touch of Weakness"]) )) then
				self.db.char[RACE] = BS["Touch of Weakness"]
			elseif ( findSpell(BS["Shadowguard"]) ) then
				self.db.char[RACE] = BS["Shadowguard"]
			end
		end
	end
	
	if ( not self.db.char.talent ) then
		self:CHARACTER_POINTS_CHANGED()
	end
end

function SacredBuff:OnEnable()
	local _, playerClass = UnitClass("player")
	if ( playerClass ~= "PRIEST" ) then
		self:ToggleActive(false)
	end
	
	self:LEARNED_SPELL_IN_TAB()
	self:BAG_UPDATE()
	
	self:RegisterBucketEvent("BAG_UPDATE", 2)
	self:RegisterBucketEvent("CHARACTER_POINTS_CHANGED", 1)
	self:RegisterEvent("LEARNED_SPELL_IN_TAB")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "ToggleCombat")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "ToggleCombat")
	
	self:UpdateButtons()
end

function SacredBuff:OnDisable()
	self.frame:Hide()
end


--[[---------------------------------------------------------------------------------
 EVENT HANDLERS
------------------------------------------------------------------------------------]]
function SacredBuff:BAG_UPDATE()
	self.db.char.candleCount = finditem(17029, true, true)
	if ( self.frame.main:GetText() ~= self.db.char.candleCount ) then
		self.frame.main:SetText(self.db.char.candleCount)
	end
	if ( not self.mounts ) then
		self.mounts = {}
	else
		self.mounts = deleteTable(self.mounts, true)
	end
	if ( GetRealZoneText() == L["Ahn'Qiraj"] ) then
		self.mounts = self:FindMounts(L["Bug_Mounts"])
	else
		self.mounts = self:FindMounts(L["Reg_Mounts"])
	end
end

function SacredBuff:LEARNED_SPELL_IN_TAB()
	for spell, _ in pairs(self.db.char.spells) do
		self.db.char.spells[spell] = findSpell(spell)
	end
end

function SacredBuff:CHARACTER_POINTS_CHANGED()
	if ( UnitLevel("player") < 40 ) then
		return
	elseif ( talentfinder(1, 15, BS["Power Infusion"]) ) then
		return
	elseif ( talentfinder(2, 16, BS["Lightwell"]) ) then
		return
	elseif ( talentfinder(3, 16, BS["Shadowform"]) ) then
		return
	end
end

function SacredBuff:ToggleCombat()
	self.inCombat = not self.inCombat
end


--[[---------------------------------------------------------------------------------
 MOUNT-RELATED FUNCTIONS
------------------------------------------------------------------------------------]]
function SacredBuff:UseMount()
	local isOnMount = ismounted()
	if ( isOnMount ) then
		CancelPlayerBuff(isOnMount)
	end
	local m
	if ( table.getn(self.mounts) == 0 ) then
		self:Print(L["There is no mount in your inventory."])
		return
	else
		math.randomseed(math.random(1337) + GetTime() * 1000)
		m = self.mounts[math.random(table.getn(self.mounts))]
	end
	UseContainerItem(m[1], m[2])
end

function SacredBuff:FindMounts(tbl)
	local list = {}
	for b = 0, NUM_BAG_FRAMES do
		for s = 1, GetContainerNumSlots(b) do
			local itemLink = GetContainerItemLink(b, s) or ""
			for _, v in ipairs(tbl) do
				if ( string.find(itemLink, v) ) then
					table.insert(list, { b, s })
				end
			end
		end
	end
	return list
end


--[[---------------------------------------------------------------------------------
 OTHER MISCELLANEOUS YET REALLY NEAT-O FUNCTIONS
------------------------------------------------------------------------------------]]
function SacredBuff:UpdateSpellIcons()
	local f = self.frame
	local spell
	if ( self.db.char.groupMode ) then
		if ( f.spirit ) then
			f.spirit.icon:SetTexture(BS:GetSpellIcon("Prayer of Spirit"))
			f.spirit:SetScript("OnClick", function()
				local onSelf = false
				if ( arg1 == "RightButton" ) then
					onSelf = true
				end
				CastSpellByName(BS["Prayer of Spirit"], onSelf)
			end)
		end
		if ( f.fort ) then
			f.fort.icon:SetTexture(BS:GetSpellIcon("Prayer of Fortitude"))
			f.fort:SetScript("OnClick", function()
				local onSelf
				if ( arg1 == "RightButton" ) then
					onSelf = true
				end
				CastSpellByName(BS["Prayer of Fortitude"], onSelf)
			end)
		end
		if ( f.shadow ) then
			f.shadow.icon:SetTexture(BS:GetSpellIcon("Prayer of Shadow Protection"))
			f.shadow:SetScript("OnClick", function()
				local onSelf
				if ( arg1 == "RightButton" ) then
					onSelf = false
				end
				CastSpellByName(BS["Prayer of Fortitude"], onSelf)
			end)
		end
		return
	end
	if ( f.spirit ) then
		f.spirit.icon:SetTexture(BS:GetSpellIcon("Divine Spirit"))
		f.spirit:SetScript("OnClick", function()
			local onSelf
			if ( arg1 == "RightButton" ) then
				onSelf = true
			end
			CastSpellByName(BS["Divine Spirit"], onSelf)
		end)
	end
	if ( f.fort ) then
		f.fort.icon:SetTexture(BS:GetSpellIcon("Power Word: Fortitude"))
		f.fort:SetScript("OnClick", function()
			local onSelf
			if ( arg1 == "RightButton" ) then
				onSelf = true
			end
			CastSpellByName(BS["Power Word: Fortitude"], onSelf)
		end)
	end
	if ( f.shadow ) then
		f.shadow.icon:SetTexture(BS:GetSpellIcon("Shadow Protection"))
		f.shadow:SetScript("OnClick", function()
			local onSelf
			if ( arg1 == "RightButton" ) then
				onSelf = true
			end
			CastSpellByName(BS["Shadow Protection"], onSelf)
		end)
	end
end

function SacredBuff:OnClick()
	if ( arg1 == "RightButton" ) then
		local x, y = finditem(6948, true)
		if ( y ) then
			UseContainerItem(x, y)
			return true
		end
		SacredBuff:Print(L["There is no Hearthstone in your inventory."])
	else
		self.db.char.groupMode = not self.db.char.groupMode
		self:UpdateSpellIcons()
	end
end